#include <Arduino.h>
#include <SPI.h>
#include <SoftwareSerial.h>
#include "BluetoothSerial.h"
#include <TinyGPS++.h>

#define GPS_BAUDRATE            9600
static const int RXPin = 16, TXPin = 17;
char cmd,cmdOld;
int userSpeed=0;
int maxSpeed=0;
byte satellitesVis=0;
bool speedActive=false;
long GPSLat=0;
long GPSLong=0;
double GPSHeight=0;
TinyGPSPlus gps;
SoftwareSerial serialGps(RXPin, TXPin);



BluetoothSerial serialBT;

#if !defined(CONFIG_BT_ENABLED) || !defined(CONFIG_BLUEDROID_ENABLED)
#error Bluetooth is not enabled! Please run `make menuconfig` to and enable it
#endif

 void readGPSSpeed();
 void satellites();
 void displayPositionData();

void setup() 
  {
    Serial.begin(115200);
    serialGps.begin(GPS_BAUDRATE);
    Serial.println(F("GPS Starting"));
    Serial.println("Starting Bluetooth");
    serialBT.begin("RC_Speedo");
    Serial.println("");
    cmd='0';
    cmdOld='0';
  }

void loop()
  {
       // read BT
      if(serialBT.available())
          {
            cmd = serialBT.read();
          
            // Start data logging
            if(cmd=='1')
              {
                      serialBT.println("Selection=1");
                      serialBT.println("GPS Speedo Started");
                      speedActive=true;
                      while(speedActive==true)
                        {                     
                          readGPSSpeed();
                          Serial.print("User Speed =");
                          Serial.println(userSpeed);
                          serialBT.print(userSpeed);
                          serialBT.println("MPH");
                          cmd=serialBT.read();
                          if(cmd=='2')
                            {
                              speedActive=false;
                              serialBT.println("GPS Stopped");
                              Serial.println("GPS Stopped");
                            }
                        } 
                  serialBT.println("GPS Speedo Stopped");
                  cmd='0';                   
              }
            // stop data logging
              if(cmd=='2')
                {
                    serialBT.println("Selection=2");
                    serialBT.println("GPS not not started");
                    Serial.println("Selection=2");
                    Serial.println("GPS Not Started");
                    cmd='0';   
                }

                if(cmd=='3')
                {
                                    //BT
                  serialBT.println("Selection=3");
                  serialBT.print("MaxSpeed=");
                  serialBT.print(maxSpeed);
                  serialBT.println("MPH");
                  //SERIAL
                  Serial.println("Selection=1");
                  Serial.print("MaxSpeed=");
                  Serial.print(maxSpeed);
                  Serial.println("MPH");

                  cmd='0';
                }
            if(cmd=='s')
              {
                satellites();
                cmd='0';
              }    

            if(cmd=='p')
              {
                displayPositionData();
                cmd='0';  
              }

            if(cmd=='9')
                {
                  maxSpeed=0;

                  //BT
                  serialBT.println("---------------");
                  serialBT.println("RESET MAX SPEED");
                  serialBT.println("---------------");
                  //SERIAL
                  Serial.println("---------------");
                  Serial.println("RESET MAX SPEED");
                  Serial.println("---------------");
                }
      // update the max speed if the speed is higher
            if(userSpeed>=maxSpeed)
            {
              maxSpeed=userSpeed;
            }
  
          }       
  }     //end loop

//------------------- funtions ------------------------

 void readGPSSpeed()
    {
        if (serialGps.available())
          {
            if(gps.encode(serialGps.read()))
              {
                if(gps.speed.isValid())
                  {
                    userSpeed = gps.speed.mph();

                    if(userSpeed>maxSpeed)
                      {
                        maxSpeed=userSpeed;
                      }
                  }
                else 
                  {
                     serialBT.println("SPEED INVALID GPS ERROR"); 
                     Serial.println("SPEED INVALID GPS ERROR"); 
                  }
              }
          }
          
    }
//---------------------------------------------------------------
void satellites()  // sat count 
  {
      if (serialGps.available())
          {
            if(gps.encode(serialGps.read()))
              {
                if(gps.satellites.isValid())
                  {
                      serialBT.print("satellites=");
                      serialBT.println(gps.satellites.value());
                      Serial.print("satellites=");
                      Serial.println(gps.satellites.value());
                  }
                else 
                  {
                     serialBT.println("SAT INVALID GPS ERROR"); 
                     Serial.println("SAT INVALID GPS ERROR"); 
                  }
              }
          }
        else 
        {
            serialBT.println("SS unavailable"); 
            Serial.println("SS unavailable");
        }
 
  }

//------------------------------------------------------------------
void displayPositionData()
  {

         if (serialGps.available() > 0)
          {
              
                
                    Serial.print("lat=");
                    Serial.println(gps.location.lat());
                    
                    serialBT.print("lat=");
                    serialBT.println(gps.location.lat());

                    Serial.print("long=");
                    Serial.println(gps.location.lng());

                    serialBT.print("long=");
                    serialBT.println(gps.location.lng());
                
                    Serial.print("alt  ");
                    Serial.println(gps.altitude.meters());
                    
                    serialBT.print("alt ");
                    serialBT.println(gps.altitude.meters());
                    
          }
          else 
            {
                  Serial.println("GPS NOT AVAILABLE to get pos");
                  serialBT.println("GPS NOT AVAILABLE to get pos");
            }
          

      
  }